#cs ----------------------------------------------------------------------------
 FastFind Version: 1.2
 Author:         FastFrench
 
 AutoIt Version: 3.3.6.1
 
 Script Function:
	All FastFind.dll wrapper functions.
    This dll provides optimized screen processing. All actions implies at least 2 dll calls : one to copy the screen data into memory, and a second to make specific action.
	This way, you can make several actions even faster when they apply to the same screen data.

 Functions exported in FastFind.dll 1.2 are :

   	SnapShot (Makes captures of the screen, a Window - partial or full - into memory, required before using any of the following.)

	ColorPixelSearch (Search the closest pixel with a given color)
	ColorCount (Count how many pixels with a given color exist in the SnapShot.)

	HasChanged (Says if two snapshots are exactly the same or not. Usefull to check if some changes occured in a given area. )
	LocalizeChanges (Same has HasChanged, but returns precisely the smallest rectangle that includes all the changes, and the number of pixels that are different).

	GetPixel (Gives the color of a pixel in the SnapShot. Much faster than PixelGetColor if you use this a lot.)

	AddColor
	RemoveColor (Those 3 functions allow management of a list of colors, instead of using only one)
	ResetColors (Tou can have up to 1024 colors active in the list)

	AddExcludedArea (Those functions add exceptions in the processing of screen areas, with rectangles that are ignored)
	ResetExcludedAreas (You can have up to 1024 areas in the list)
	IsExcluded

	ColorsSearch (Close to ColorSearch, except that instead of a single color, all colors of the current list are in use)
	ColorsPixelSearch (Similar to ColorPixelSearch, except that instead of a single color, all colors of the current list are in use)

	ColorSearch (it's the most versatile and powerful function here : you can, at the same time, check for as many colors as you want,
	             with possibly a "ShadeVariation",  multiple subareas to ignore... will Find the closest spot with a all specified criteria.
				 The spot is a square area of NxN pixels with at least P pixels that are as close as allowed by ShadeVariation
				 from any of the colors in the list).

	GetLastErrorMsg
	FFVersion

#ce ----------------------------------------------------------------------------


; Variables globales - paramtrage modifiable par l'utilisateur
global $FFDefaultSnapShot = 0 	 ; N du SnapShot si aucun n'est indiqu


global $FFDefautDebugMode = 0x27 ; Mettre 0 (ou 0xFF2F en Beta) pour du code en utilisation relle, 0x27 est pas mal pour dbuter en dveloppement. Pour les premiers essais, essayez avec 0x002F (affiche des MessageBox de synthse aprs chaque fonction importante)


; Variables globale  ne pas modifier
global $FFDllHandle = -1
global $FFLastSnap = 0
global const $FFNbSnapMax = 1024
global $FFLastSnapStatus[$FFNbSnapMax]

InitFFDll()



; Chargement et dchargement de FF.dll
; ------------------------------------------
Func InitFFDll()
for $i = 0 To $FFNbSnapMax-1
	$FFLastSnapStatus[$i] = 0
	Next
If @AutoItX64 Then
	Global $DllName = "FastFind64.dll"
Else
	Global $DllName = "FastFind.dll"
Endif
	$FFDllHandle = DllOpen($DllName)
If $FFDllHandle=-1 Then
	$FFDllHandle=$DllName
	MsgBox(0,"Error","Failed to load "&$DllName&", application probably won't properly work. "&@LF&"Check if the file FastFind.dll is installed near this script")
	Return
	EndIf
FFSetDebugMode($FFDefautDebugMode)
EndFunc

Func CloseFFDll()
	If $FFDllHandle<>-1 Then DllClose($FFDllHandle)
EndFunc

; Dtermine le mode de debuggage.
; ---------------------------------------------------------------------
;  Les 4 bits de poids faible dterminent les canaux de debuggage activs, ils ont la signification suivante :
;     0x00 = sans debug,
;     0x01 = Informations envoyes vers la console (RequireAdmin),
;     0x02 = Information debug envoyes vers un fichier (trace.txt)
;	  0x04 = Affichage graphique des points / zones dtects
;     0x08 = Affichage de MessageBox (bloquant)
;     A noter qu'en cas d'erreur, une MessageBox sera affiche dans la DLL si DebugMode>0

; Les 4 bits suivants permettent de filtrer selon la provenance du message de debug
;     0x0010 // Exclut les traces internes de la DLL
;     0x0020 // Exclut les traces internes dtailles de la DLL
;     0x0040 // Exclut les traces externes (celles de l'application)
;     0x0080 // Message d'erreur (prioritaire)
;
; Les erreurs (graves) sont affiches sur tous les canaux disponibles (fichier, console et MessageBos) si $DebugMode > 0
;
; Proto fonction C : void SetDebugMode(int NewMode)
Func FFSetDebugMode($DebugMode)
	DllCall($FFDllHandle, "none", "SetDebugMode", "int", $DebugMode)
EndFunc

; La DLL expose galement ses fonction de dbuggage, permettant  l'application AutoIt d'en tirer parti
Func FFTrace($DebugString)
	DllCall($FFDllHandle, "none", "DebugTrace", "str", $DebugString)
EndFunc


; Cette fonction permet de traiter les erreurs (le texte apparait dans le fichier de traces, la console et une MessageBox si $DebugMode>0)
Func FFTraceError($DebugString)
	DllCall($FFDllHandle, "none", "DebugError", "str", $DebugString)
EndFunc


; Fixe la fentre courante  utiliser
; -----------------------------------
; Par dfaut, c'est tout l'cran qui est utilis. Il est possible de choisir une fentre en particulier.
; Ce choix n'a d'influence directe que sur "SnapShot". Pour les autres oprations, c'est la fentre
; courante au moment de la prise du SnapShot qui compte.
; Si $WindowsHandle = 0, c'est tout l'cran - GetDesktopWindow()
;
; Proto fonction C : void SetHWnd(HWND NewWindowHandle);
Func FFSetWnd($WindowHandle)
	DllCall($FFDllHandle, "none", "SetHWnd", "HWND", $WindowHandle)
EndFunc


Func FFSetDefaultSnapShot($NewSnapShot)
	$FFDefaultSnapShot = $NewSnapShot
EndFunc

; Gestion de la liste des couleurs
; Quand un paramtre de Couleur est propos dane une fonction, la valeur -1 signifie que toutes

; Ajoute une ou plusieurs couleurs dans la liste gre par FastFind
; Proto fonction C : int AddColor (COLORREF NewColor)
Func FFAddColor($NewColor)
	local $res
	if (IsArray($NewColor)) Then
		For $Color In $NewColor
			$res = DllCall($FFDllHandle, "int", "AddColor", "int", $Color)
		Next
	Else
		$res = DllCall($FFDllHandle, "int", "AddColor", "int", $NewColor)
	EndIf
	if IsArray($res) Then return $res[0]
	return $res
EndFunc

; Retire une couleur (si elle existe) de la liste des couleurs
;
; Proto fonction C : int RemoveColor (COLORREF NewColor)
Func FFRemoveColor($OldColor)
	local $res = DllCall($FFDllHandle, "int", "RemoveColor", "int", $OldColor)
	if IsArray($res) Then return $res[0]
	return $res
EndFunc

; Vide totalement la liste des couleurs
;
; Proto fonction C : int ResetColors ()
Func FFResetColors()
	DllCall($FFDllHandle, "none", "ResetColors")
EndFunc


; Les zones d'exclusion permettent de restreindre les recherches effectues avec toutes les fonctions
; de recherche
; Il est possible d'avoir jusqu' 1024 rectangles d'exclusion, permettant ainsi de retirer prcisment
; n'importe quelle zone des recherches. Par exemple, avec flash, le curseur de la souris apparait dans
; la capture d'cran (contrairement aux curseurs grs par Windows). Vous pouvez donc utiliser une
; zone d'exclusion constitue d'aprs la position de la souris, afin que le curseur n'influe pas sur le
; rsultat des recherches.
;
; Ajoute une zone d'exclusion
;
; Proto fonction C : void WINAPI AddExcludedArea(int x1, int y1, int x2, int y2)
Func FFAddExcludedArea($x1, $y1, $x2, $y2)
	local $Res = DllCall($FFDllHandle, "int", "FFAddExcludedArea", "int", $x1, "int", $y1, "int", $x2, "int", $y2)
	if IsArray($res) Then return $res[0]
	return $res
EndFunc

; Supprime la liste de toutes les zones d'exclusion
;
; Proto fonction C : void WINAPI ResetExcludedAreas()
Func FFResetExcludedAreas()
	DllCall($FFDllHandle, "void", "ResetExcludedAreas")
EndFunc

; Parcourt la liste des zones d'exclusion pour dterminer si le point pass en paramtre pour la fentre indique
; est exclu des recherches ou pas.
;
; Proto fonction C : bool WINAPI IsExcluded(int x, int y, HWND hWnd)
Func FFIsExcluded($x, $y, $hWnd)
	local $Res = DllCall($FFDllHandle, "BOOLEAN", "IsExcluded", "int", $x, "int", $y, "HWND", $hWnd)
	if IsArray($res) Then return $res[0]
	return $res
EndFunc


; Fonction FFSnapShot - Cette fonction permet de faire une copie de l'cran, d'une fentre ou seulement d'une partie en mmoire
;							- Toutes les autres fonctions de FF fonctionnant  partir de la mmoire, il faut commencer par utiliser FFSnapShot
; Il est possible d'effectuer plusieurs captures diffrentes, et de travailler sur n'importe laquelle par la suite.
; Certaines fonctions permettent mme de comparer les captures entre elles
; En entre :
;    la zone  capturer (en coordonnes relatives aux limites de la fentre, si un handle de fentre non nul est indiqu) [optionnel, l'cran entier par dfaut]
;        Si la zone indique est 0,0,0,0 alors c'est la totalit de la fentre qui sera capture
;    le N de SnapShot  utiliser (optionnel, le dernier utilis par dfaut, 0 initialement)
;	 un handle de fentre [optionnel, le mme cran que la fois prcdente par dfaut. Initialement, tout l'cran.]
;
; Attention : les donnes graphiques tant conserves en mmoire, l'utilisation de cette fonction consomme de la mmoire. Il faut compter environ 1,8 Mo de RAM pour une capture 800x600.
; 			  c'est pourquoi il faut de prfrence rutiliser toujours les mmes N de SnapShot. Nanmoins, il est possible de mmoriser jusqu' 1024 crans diffrents.

; Valeurs de retour : En cas d'chec, renvoie 0 et positionne @Error.
;					  En cas de succs, renvoie 1
; Proto fonction C : int WINAPI SnapShot(int aLeft, int aTop, int aRight, int aBottom, int NoSnapShot)
Func FFSnapShot($Left=0, $Top=0, $Right=0, $Bottom=0, $NoSnapShot=$FFDefaultSnapShot, $WindowHandle=-1)
	if ($WindowHandle <> -1) Then FFSetWnd($WindowHandle)
	$FFDefaultSnapShot = $NoSnapShot ; On mmorise le no du SnapShot utilis, cela restera le SnapShop par dfaut pour les prochains appels
	local $Res = DllCall($FFDllHandle, "int", "SnapShot", "int", $Left, "int", $Top, "int", $Right, "int", $Bottom, "int", $NoSnapShot)
	If ( ((Not IsArray($Res)) AND ($Res=0)) OR $Res[0]=0) Then
		MsgBox(0, "FFSnapShot", "SnapShot ("&$Left&","&$Top&","&$Right&","&$Bottom&","&$NoSnapShot&","&Hex($WindowHandle,8)&") failed ")
		if (IsArray($Res)) Then
			MsgBox(0, "FFSnapShot Error", "IsArray($Res):"&IsArray($Res)&" - Ubound($Res):"&UBound($Res)&" - $Res[0]:"&$Res[0])
		else
			MsgBox(0, "FFSnapShot Error", "IsArray($Res):"&IsArray($Res)&" - $Res:"&$Res)
		EndIf
		$FFLastSnapStatus[$NoSnapShot] = -1
		SetError(2)
		Return False
		EndIf
	$FFLastSnapStatus[$NoSnapShot] = 1
	$FFLastSnap  = $NoSnapShot
	Return True
EndFunc


; Fonction interne, premiers traitements utiliss dans une partie des fonction -  ne pas utiliser directement
Func SnapShotPreProcessor($Left, $Top, $Right, $Bottom, $ForceNewSnap, $NoSnapShot, $WindowHandle)
	; Si on impose une nouvelle capture ou si aucun SnapShot valide n'a dj t effectu pour ce N
	if ($ForceNewSnap OR $FFLastSnapStatus[$NoSnapShot] <> 1) Then return FFSnapShot($Left, $Top, $Right, $Bottom, $NoSnapShot, $WindowHandle)
	Return True
EndFunc

; Fonction FFNearestPixel - Cette fonction fonctionne un peu comme PixelSearch,
;                                 sauf qu'elle renvoie non pas le premier pixel trouv,
;                                 mais le plus proche de PoxX/PoxY
; Valeurs de retour : En cas d'chec, renvoie 0 et positionne @Error.
;					  En cas de succs, un tableau de 2 lments : 1/ coordonne X du pixel trouv le plus proche, 2/ coordonne Y du pixel trouv le plus proche
; Exemple : pour trouver le pixel de couleur 0x00AB0C45 le plus proche possible du joueur situ en 500, 500
; $Res = FFNearestPixel( 0, 0, 1023, 767, 1, 500, 500, 0x00AB0C45)
; If not @Error Then MsgBox(0, "Ressource", "Trouve en "&$PosX&","&$PosY)
;
; Proto fonction C : int WINAPI ColorPixelSearch(int &XRef, int &YRef, int ColorToFind, int NoSnapShot)
Func FFNearestPixel($Left, $Top, $Right, $Bottom, $PosX, $PosY, $Color, $ForceNewSnap=true, $NoSnapShot=$FFLastSnap, $WindowHandle=-1)
 	;local $NoSnapShot = 2 ; Slot utilis pour les captures d'cran (entre 0 et 3), choisi arbitrairement
	if Not SnapShotPreProcessor($Left, $Top, $Right, $Bottom, $ForceNewSnap, $NoSnapShot, $WindowHandle) Then
		SetError(2)
		Return False
		EndIf
	local $Result = DllCall($FFDllHandle, "int", "ColorPixelSearch", "int*", $PosX, "int*",$PosY, "int", $Color, "int", $NoSnapShot)
	If ( Not IsArray($Result) OR $Result[0]<>1) Then
		SetError(1)
		Return False
		EndIf
	local $CoordResult[2] = [$Result[1], $Result[2]] ; PosX, PosY
	return $CoordResult
EndFunc

; Fonction FFNearestSpot - Cette fonction permet de trouver, parmi toutes les zone (ou "Spots") contenant
;						         un nombre minimum de pixels d'une certaines couleur, celle qui est la plus proche d'un point (position du joueur par exemple).
; Valeurs de retour : En cas d'chec, renvoie 0 et positionne @Error.
;					 En cas de succs, un tableau de 3 lments : 1/ coordonne X du spot le plus proche, 2/ coordonne Y du spot le plus proche, 3/ Nombre de pixels trouvs dans le spot le plus proche
; Par exemple, supposons que vous voulez dtecter un cercle bleu (Color = 0x000000FF) partiellement occult, de diamtre 32 pixels (disons avec au moins 45 pixels de la bonne couleur),
; le plus proche possible de la position x=198 et y=543, dans un cran complet en rsolution 1024x768; alors la fonction est appele ainsi :
; $Res = FFNearestSpot(0, 0, 1023, 767, 45, 198, 543, 0x000000FF)
; If not @Error Then MsgBox(0, "Blue Circle", "Le cercle bleu le plus proche de la position 198, 543 se trouve en "&$PosX&","&$PosY&@LF&" et contient "&$NbPixel&" pixels bleus")
;
; Proto fonction C : int WINAPI GenericColorSearch(int SizeSearch, int &NbMatchMin, int &XRef, int &YRef, int ColorToFind, int ShadeVariation, int NoSnapShot)
Func FFNearestSpot($Left, $Top, $Right, $Bottom, $SizeSearch, $NbPixel, $PosX, $PosY, $Color, $ShadeVariation=0, $ForceNewSnap=true, $NoSnapShot=$FFLastSnap, $WindowHandle=-1)
	if Not SnapShotPreProcessor($Left, $Top, $Right, $Bottom, $ForceNewSnap, $NoSnapShot, $WindowHandle) Then
		SetError(2)
		Return False
		EndIf
	local $Result = DllCall($FFDllHandle, "int", "GenericColorSearch", "int", $SizeSearch, "int*", $NbPixel, "int*", $PosX, "int*",$PosY, "int", $Color, "int", $ShadeVariation, "int", $NoSnapShot)
	If ((Not IsArray($Result)) OR $Result[0]<>1) Then
		SetError(1)
		Return False
		EndIF
	local $CoordResult[3] = [$Result[3], $Result[4], $Result[2]] ; PosX, PoxY, Nombre de pixels
	return $CoordResult
EndFunc


; Fonction FFColorCount - Cette fonction compte le nombre de pixels ayant la couleur indique, exactes ou approches (ShadeVariation).
;
; Proto C : int WINAPI ColorCount(int ColorToFind, int NoSnapShot, int ShadeVariation)
Func FFColorCount($Left, $Top, $Right, $Bottom, $ColorToCount, $ShadeVariation=0, $ForceNewSnap=true, $NoSnapShot=$FFLastSnap, $WindowHandle=-1)
	if Not SnapShotPreProcessor($Left, $Top, $Right, $Bottom, $ForceNewSnap, $NoSnapShot, $WindowHandle)  Then
		SetError(2)
		Return False
		EndIf
	$Result = DllCall($FFDllHandle, "int", "ColorCount", "int", $ColorToCount, "int", $ShadeVariation)
	If (Not IsArray($Result)) Then Return False
	Return $Result[0]
EndFunc

; Fonction FFIsDifferent - Cette fonction compare deux SnapShots et indique si ils ont chang.
;
; Proto C : int WINAPI HasChanged(int NoSnapShot, int NoSnapShot2)
Func FFIsDifferent($NoSnapShot1, $NoSnapShot2)
	$Result = DllCall($FFDllHandle, "int", "HasChanged", "int", $NoSnapShot1, "int", $NoSnapShot2)
	If (Not IsArray($Result)) Then Return False
	Return $Result[0]
EndFunc


; Fonction FFLocalizeChanges - Cette fonction compare deux SnapShots et prcise le nombre de pixels diffrent ainsi que le plus petit rectangle contenant tous les changements.
; En cas d'chec, @Error = 1 et retourne 0
; En cas de diffrences constates,	renvoie un tableau de 5 lments ainsi form :
; 		[0] : limite gauche du rectangle
; 		[1] : limite haute du rectangle
; 		[2] : limite droite du rectangle
; 		[3] : limite basse du rectangle
;	    [4] : Nombre de pixels ayant chang
; Proto C : int WINAPI LocalizeChanges(int NoSnapShot, int NoSnapShot2, int &xMin, int &yMin, int &xMax, int &yMax, int &nbFound)
Func FFLocalizeChanges($NoSnapShot1, $NoSnapShot2)
	$Result = DllCall($FFDllHandle, "int", "LocalizeChanges", "int", $NoSnapShot1, "int", $NoSnapShot2, "int*", 0, "int*", 0, "int*", 0, "int*", 0, "int*", 0)
	If ((Not IsArray($Result)) OR $Result[0]<>1) Then
		SetError(1)
		Return False
		EndIF
	local $TabRes[5] = 	[$Result[3], $Result[4], $Result[5], $Result[6], $Result[7]];
	Return $TabRes
EndFunc


; Fonction FFGetPixel - Cette fonction est quivalente  PixelGetColor, mais fonctionne  partir d'un SnapShot.
;                       Pour que cette fonction reste aussi rapide que possible, la cration d'un SnapShot pralable doit tre explicite
;
; Proto C : int WINAPI FFgetPixel(int X, int Y, int NoSnapShot)
Func FFGetPixel($x, $y, $NoSnapShot=$FFLastSnap)
	$Result = DllCall($FFDllHandle, "int", "GetPixel", "int", $x, "int", $y, "int", $NoSnapShot)
	If ( (Not IsArray($Result)) or ($Result[0]=-1) ) Then 
		SetError(2)
		Return -1	
		EndIf
	Return $Result[0]
EndFunc

; Fonction FFGetVersion - Cette fonction retourne le N de version de la Dll FastFind
;
; Proto C : LPCTSTR WINAPI FFVersion(void)
Func FFGetVersion()
	$Result = DllCall($FFDllHandle, "str", "FFVersion")
	If ( (Not IsArray($Result))  ) Then 
		SetError(2)
		Return "???"	
		EndIf
	Return $Result[0]
EndFunc

; Fonction FFGetVersion - Cette fonction retourne le dernier message d'erreur
;
; Proto C : LPCTSTR WINAPI GetLastErrorMsg(void)
Func FFGetLastError()
	$Result = DllCall($FFDllHandle, "str", "GetLastErrorMsg")
	If ( (Not IsArray($Result))  ) Then 
		SetError(2)
		Return ""	
		EndIf
	Return $Result[0]
EndFunc
